/**
 * vue-promised v0.1.0
 * (c) 2018 Eduardo San Martin Morote <posva13@gmail.com>
 * @license MIT
 */

'use strict';

function assert (condition, message) {
  if (!condition) {
    throw new Error(("[vue-promised] " + message))
  }
}

var index = {
  name: 'Promised',
  props: {
    promise: Promise,
    promises: Array,
  },

  data: function () { return ({
    resolved: false,
    data: null,
    error: null,
  }); },

  render: function render (h) {
    if (this.error instanceof Error || (this.error && this.error.length)) {
      assert(
        this.$scopedSlots && this.$scopedSlots.error,
        'Provide exactly one scoped slot named "error" for the rejected promise'
      );
      return this.$scopedSlots.error(this.error)
    } else if (this.resolved) {
      assert(
        this.$scopedSlots && this.$scopedSlots.default,
        'Provide exactly one default scoped slot for the resolved promise'
      );
      return this.$scopedSlots.default(this.data)
    } else {
      assert(
        this.$slots.default && this.$slots.default.length === 1,
        'Provide exactly one default slot with no `slot-scope` for the pending promise'
      );
      return this.$slots.default[0]
    }
  },

  watch: {
    promise: {
      handler: function handler (promise) {
        var this$1 = this;

        if (!promise) { return }
        this.resolved = false;
        this.error = null;
        promise
          .then(function (data) {
            if (this$1.promise === promise) {
              this$1.resolved = true;
              this$1.data = data;
            }
          })
          .catch(function (err) {
            if (this$1.promise === promise) { this$1.error = err; }
          });
      },
      immediate: true,
    },

    promises: {
      handler: function handler (promises) {
        var this$1 = this;

        if (!promises) { return }
        this.resolved = false;
        this.error = [];
        this.data = [];
        promises.forEach(function (p) {
          p
            .then(function (data) {
              if (this$1.promises === promises) {
                this$1.resolved = true;
                this$1.data.push(data);
              }
            })
            .catch(function (err) {
              if (this$1.promises === promises) { this$1.error.push(err); }
            });
        });
      },
      immediate: true,
    },
  },
};

module.exports = index;
